"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.installPulumi = exports.needsPulumi = void 0;
const os = __importStar(require("os"));
const tar = __importStar(require("tar"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const jszip_1 = __importDefault(require("jszip"));
const axios_1 = __importDefault(require("axios"));
const execa_1 = require("@esm2cjs/execa");
// TODO: Obtain this information from the variable provided from the cli.
const BINARY_DIR = (() => {
    const binDir = path.resolve(os.homedir(), ".pluto/bin");
    process.env.PATH = `${binDir}:${process.env.PATH}`;
    fs.ensureDirSync(binDir, { mode: 0o755 });
    return binDir;
})();
const PULUMI_VERSION = "v3.112.0";
async function needsPulumi() {
    const pulumiPath = path.join(BINARY_DIR, "pulumi");
    try {
        fs.accessSync(pulumiPath);
        const { stdout: output } = await (0, execa_1.execa)(pulumiPath, ["version"]);
        const version = output?.trim();
        return version !== PULUMI_VERSION;
    }
    catch (error) {
        // The file does not exist or other errors.
        return true;
    }
}
exports.needsPulumi = needsPulumi;
async function installPulumi() {
    const getOsArch = () => {
        switch (os.platform()) {
            case "darwin":
                return os.arch() === "arm64" ? "darwin-arm64" : "darwin-x64";
            case "linux":
                return os.arch() === "arm64" ? "linux-aarch64" : "linux-x64";
            case "win32":
                return "windows-x64";
            default:
                throw new Error("unsupported operating system");
        }
    };
    const getOsFileExtension = () => {
        return os.platform() === "win32" ? ".zip" : ".tar.gz";
    };
    console.info("Starting pulumi installation");
    const osArch = getOsArch();
    const fileExtension = getOsFileExtension();
    const url = `https://get.pulumi.com/releases/sdk/pulumi-${PULUMI_VERSION}-${osArch}${fileExtension}`;
    console.info("Downloading pulumi", { url });
    const response = await (0, axios_1.default)({
        method: "get",
        url: url,
        responseType: "arraybuffer",
    });
    const pulumiData = response.data;
    const tempFilePath = path.join(BINARY_DIR, `pulumi${fileExtension}`);
    await fs.writeFile(tempFilePath, pulumiData);
    if (fileExtension === ".tar.gz") {
        // Extract tar.gz using tar package
        await tar.x({
            file: tempFilePath,
            cwd: BINARY_DIR,
            strip: 1, // Remove the top-level directory
        });
    }
    else if (fileExtension === ".zip") {
        // Extract zip file using jszip.
        const zip = new jszip_1.default();
        const zipContents = await zip.loadAsync(pulumiData);
        await Promise.all(Object.keys(zipContents.files).map(async (filename) => {
            const fileData = await zipContents.files[filename].async("nodebuffer");
            // Remove the top-level directory
            const filePath = path.join(BINARY_DIR, filename.substring("pulumi/".length));
            await fs.writeFile(filePath, fileData);
        }));
    }
    else {
        throw new Error("Unsupported file extension for Pulumi package");
    }
    await fs.unlink(tempFilePath); // Remove temporary archive file
    console.info("Finished installing pulumi");
}
exports.installPulumi = installPulumi;
//# sourceMappingURL=installation.js.map